#============================================================================
def qTrapz(Func, a, b, n):
#----------------------------------------------------------------------------
#  Integrates function Func on interval [a,b] using the trapezoidal rule
#  with n integration points
#----------------------------------------------------------------------------
   h = (b-a)/(n-1)
   s = 0.5*(Func(a) + Func(b))
   for i in range(1,n-1): s += Func(a+i*h)

   return h*s

#============================================================================
def qSimpson(Func, a, b, n):
#----------------------------------------------------------------------------
#  Integrates function Func on interval [a,b] using Simpson's rule with n
#  (odd) integration points
#----------------------------------------------------------------------------
   if (n % 2 == 0): n += 1                              # increment n if even

   h = (b-a)/(n-1)
   s1 = s2 = 0e0
   for i in range(2,n-2,2): s1 += Func(a + i*h)               # odd-index sum
   for i in range(1,n-1,2): s2 += Func(a + i*h)              # even-index sum

   return (h/3)*(Func(a) + 4*s2 + 2*s1 + Func(b))

#============================================================================
def qTrapzCtrl(Func, a, b, eps = 1e-6):
#----------------------------------------------------------------------------
#  Integrates function Func on interval [a,b] with relative precision eps
#  using the adaptive trapezoidal rule
#----------------------------------------------------------------------------
   kmax = 30                            # max. no. of step halving iterations

   h = b-a; n = 1
   t0 = 0.5*h*(Func(a) + Func(b))                     # initial approximation

   for k in range(1,kmax+1):                              # step halving loop
      sumf = 0e0
      for i in range(1,n+1): sumf += Func(a+(i-0.5)*h)
      t = 0.5*(t0 + h*sumf)                               # new approximation
      if (k > 1):                                         # convergence check
         if (abs(t-t0) <= eps*abs(t)): break
         if (abs(t) <= eps and abs(t) <= abs(t-t0)): break # integral ~= 0
      h *= 0.5; n *= 2                               # halve integration step
      t0 = t

   if (k >= kmax): print("qTrapzCtrl: max. no. of iterations exceeded !")

   return t

#============================================================================
def qSimpsonCtrl(Func, a, b, eps = 1e-6):
#----------------------------------------------------------------------------
#  Integrates function Func on interval [a,b] with relative precision eps
#  using the adaptive Simpson rule
#----------------------------------------------------------------------------
   kmax = 30                            # max. no. of step halving iterations

   h = b-a; n = 1
   s0 = t0 = 0.5*h*(Func(a) + Func(b))                # initial approximation

   for k in range(1,kmax+1):                              # step halving loop
      sumf = 0e0
      for i in range(1,n+1): sumf += Func(a+(i-0.5)*h)
      t = 0.5*(t0 + h*sumf)
      s = (4*t - t0)/3                                    # new approximation
      if (k > 1):                                         # convergence check
         if (abs(s-s0) <= eps*abs(s)): break
         if (abs(s) <= eps and abs(s) <= abs(s-s0)): break # integral ~= 0
      h *= 0.5; n *= 2                               # halve integration step
      s0 = s; t0 = t

   if (k >= kmax): print("qSimpsonCtrl: max. no. of iterations exceeded!")

   return s

#============================================================================
def qRomberg(Func, a, b, eps = 1e-6):
#----------------------------------------------------------------------------
#  Integrates function Func on interval [a,b] with relative precision eps
#  using the adaptive Romberg method
#----------------------------------------------------------------------------
   kmax = 30                            # max. no. of step halving iterations
   r1 = [0]*(kmax+1)                                  # two consecutive lines
   r2 = [0]*(kmax+1)                                  # from the method table

   h = b-a; n = 1
   r1[0] = 0.5*h*(Func(a) + Func(b))                  # initial approximation
   for k in range(1,kmax+1):                              # step halving loop
      sumf = 0e0
      for i in range(1,n+1): sumf += Func(a+(i-0.5)*h)
      r2[0] = 0.5*(r1[0] + h*sumf)                        # trapezoid formula
      f = 1e0
      for j in range(1,k+1):                      # increase quadrature order
         f *= 4
         r2[j] = (f*r2[j-1] - r1[j-1])/(f-1)              # new approximation

      if (k > 1):                                         # convergence check
         if (abs(r2[k]-r1[k-1]) <= eps*abs(r2[k])): break
         if (abs(r2[k]) <= eps and abs(r2[k]) <= abs(r2[k]-r1[k-1])):break
      h *= 0.5; n *= 2                               # halve integration step
      for j in range(0,k+1): r1[j] = r2[j]                # shift table lines

   if (k >= kmax):
      print("qRomberg: max. no. of iterations exceeded !")
      k -= 1

   return r2[k]

def f(x):
    return (x**4+1)

print 'Trapezoidal rule'    
print qTrapz(f, 1.0, 2.0, 4)
print 'Simpson 1/3 rule'
print qSimpson(f, 1.0, 2.0, 4)  
print 'Adaptive trapezoidal rule'    
print qSimpsonCtrl(f, 1.0, 2.0, 1e-6)
print 'Adaptive Simpson 1/3 rule' 
print qSimpsonCtrl(f, 1.0, 2.0, 1e-6)
print 'Romberg method' 
print qRomberg(f, 1.0, 2.0, 1e-6)    
