def Gauss(a, b, n):
#----------------------------------------------------------------------------
#  Solves matrix equation a x = b by Gaussian elimination with partial
#  pivoting on columns.
#  a   - coefficient matrix (n x n); destroyed on exit
#  b   - matrix of constant terms ; solution x on exit
#  det - determinant of coefficient matrix (output).
#----------------------------------------------------------------------------
   det = 1e0
   for k in range(0,n):                               # FORWARD ELIMINATION
      amax = 0e0                                   # determine pivot row with
      for i in range(k,n):                       # max. element on column k
         if (amax < abs(a[i][k])): amax = abs(a[i][k]); imax = i
      if (amax == 0e0):
         print("Gauss: singular matrix !"); return 0e0

      if (imax != k):                           # interchange rows imax and k
         det = -det                             # to put pivot on diagonal
         for j in range(k,n): (a[imax][j],a[k][j]) = (a[k][j],a[imax][j])
         (b[imax],b[k]) = (b[k],b[imax])

      det *= a[k][k]                        # multiply determinant with pivot

      t = 1e0/a[k][k]                             # divide pivot row by pivot
      for j in range(k+1,n): a[k][j] *= t
      b[k] *= t

      for i in range(k+1,n):                        # reduce non-pivot rows
         t = a[i][k]
         for j in range(k+1,n): a[i][j] -= a[k][j]*t
         b[i] -= b[k]*t

   for k in range(n-1,-1,-1):                          # BACKWARD SUBSTITUTION
         sum = b[k]
         for i in range(k+1,n): sum -= a[k][i]*b[i]
         b[k] = sum

   return det

#============================================================================
def GaussJordan(a, b, n):
#----------------------------------------------------------------------------
#  Solves matrix equation a x = b by Gauss-Jordan elimination with partial
#  pivoting on columns.
#  a   - coefficient matrix (n x n); a^(-1) on exit
#  b   - matrix of constant terms; solution x on exit
#  det - determinant of coefficient matrix (output).
#----------------------------------------------------------------------------
   ipivot = [0]*(n)                                     # stores pivot rows

   det = 1e0
   for k in range(0,n):                               # FORWARD ELIMINATION
      amax = 0e0                                   # determine pivot row with
      for i in range(k,n):                       # max. element on column k
         if (amax < abs(a[i][k])): amax = abs(a[i][k]); imax = i
      if (amax == 0e0):
         print("GaussJordan1: singular matrix !"); return 0e0
      ipivot[k] = imax                                # store pivot row index

      if (imax != k):                           # interchange rows imax and k
         det = -det                             # to put pivot on diagonal
         for j in range(0,n): (a[imax][j],a[k][j]) = (a[k][j],a[imax][j])
         (b[imax],b[k]) = (b[k],b[imax])

      det *= a[k][k]                        # multiply determinant with pivot

      t = 1e0/a[k][k]                             # divide pivot row by pivot
      a[k][k] = 1e0                         # diagonal element of unit matrix
      for j in range(0,n): a[k][j] *= t
      b[k] *= t

      for i in range(0,n):                          # reduce non-pivot rows
         if (i != k): 
            t = a[i][k]
            a[i][k] = 0e0               # non-diagonal element of unit matrix
            for j in range(0,n): a[i][j] -= a[k][j]*t
            b[i] -= b[k]*t

   for k in range(n-1,-1,-1):                     # rearrange columns of inverse
      imax = ipivot[k]
      if (imax != k):
         for i in range(0,n): (a[i][imax],a[i][k]) = (a[i][k],a[i][imax])

   return det

#============================================================================
def GaussSeidel(a, b, x, n):
#----------------------------------------------------------------------------
#  Solves linear system a x = b by the Gauss-Seidel method.
#  Convergence is ensure by left-multiplying the system with a^T.
#  a    - coefficient matrix (n x n)
#  b    - vector of constant terms
#  x    - initial approximation of solution (input); solution (output)
#  n    - order of system
#  err  - maximum relative error of the solution components
#----------------------------------------------------------------------------
   eps = 1e-10                                 # relative precision criterion
   itmax = 1000                                       # max no. of iterations

   s = [[0]*(n) for i in range(n)]           # matrices of reduced system
   t = [0]*(n)

   for i in range(0,n):                         # matrices of normal system
      for j in range(0,i+1):                      # by multiplication with aT
         s[i][j] = 0e0                            # store result in s and t
         for k in range(0,n): s[i][j] += a[k][i]*a[k][j]
         s[j][i] = s[i][j]

      t[i] = 0e0
      for j in range(0,n): t[i] += a[j][i]*b[j]

   for i in range(0,n):                # matrices s and t of reduced system
      f = -1e0/s[i][i]; t[i] /= s[i][i]
      for j in range(0,n): s[i][j] *= f

   for k in range(1,itmax+1):                            # loop of iterations
      err = 0e0
      for i in range(0,n):
         delta = t[i]                                            # correction
         for j in range(0,n): delta += s[i][j]*x[j]
         x[i] += delta                        # new approximation to solution
         if (x[i]): delta /= x[i]                            # relative error
         if (abs(delta) > err): err = abs(delta)            # maximum error

      if (err <= eps): break                              # check convergence

   if (k > itmax): print("GaussSeidel: max. no. of iterations exceeded !")

   return err

a= [[0 for x in range(2)] for x in range(2)] 
a[0][0] = 1
a[0][1] = 2
a[1][0] = 2
a[1][1] = 3
b=[0 for x in range(2)] 
b[0] = 3
b[1] = 5
print 'Gaussian elimination with partial pivoting'
print 'det=',Gauss(a, b, 2)
print b
a[0][0] = 1
a[0][1] = 2
a[1][0] = 2
a[1][1] = 3
b[0] = 3
b[1] = 5
print 'Gauss-Jordan elimination with partial'
print 'det=',GaussJordan(a, b, 2)
print b
a[0][0] = 1
a[0][1] = 2
a[1][0] = 2
a[1][1] = 3
x=[0 for y in range(2)] 
x[0] = 0
x[1] = 0
b[0] = 3
b[1] = 5
print 'Gauss-Seidel method'
GaussSeidel(a, b, x, 2)
print x